package com.huigou.topsun.proofing.application.impl;

import com.huigou.data.query.model.QueryDescriptor;
import com.huigou.topsun.proofing.application.WorkInstructionApplication;
import com.huigou.topsun.proofing.domain.vo.WorkInstructionVo;
import com.huigou.topsun.proofing.domain.workInstruction.model.*;
import com.huigou.topsun.proofing.domain.workInstruction.query.WorkInstructionBaseInfoQueryRequest;
import com.huigou.topsun.proofing.repository.workInstruction.*;
import com.huigou.topsun.util.MyBaseUtil;
import com.huigou.uasp.bmp.common.application.BaseApplication;
import com.huigou.util.SDO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Map;
import java.util.stream.Stream;

@Service
public class WorkInstructionApplicationImpl extends BaseApplication implements WorkInstructionApplication {

    @Autowired
    WorkInstructionRepository workInstructionRepository;
    @Autowired
    WorkInstructionPdPublishedConfRepository workInstructionPdPublishedConfRepository;
    @Autowired
    WorkInstructionMaterialInfoRepository workInstructionMaterialInfoRepository;
    @Autowired
    WorkInstructionOptionalAttributeRepository workInstructionOptionalAttributeRepository;
    @Autowired
    WorkInstructionPrdLookedRepository workInstructionPrdLookedRepository;
    @Autowired
    WorkInstructionProductLossRepository workInstructionProductLossRepository;

    /**
     * 添加作业指导书及相关信息
     * @param sdo
     * @return
     */
    @Override
    @Transactional(propagation = Propagation.REQUIRES_NEW)
    public String saveWorkInstruction(SDO sdo) {
        WorkInstructionBaseInfo workInstruction = sdo.toObject(WorkInstructionBaseInfo.class);
        WorkInstructionPdPublishedConf workInstructionPdPublishedConf = sdo.toObject(WorkInstructionPdPublishedConf.class);
        WorkInstructionMaterialInfo workInstructionMaterialInfo = sdo.toObject(WorkInstructionMaterialInfo.class);
        WorkInstructionOptionalAttribute workInstructionOptionalAttribute = sdo.toObject( WorkInstructionOptionalAttribute.class);
        WorkInstructionPrdLooked workInstructionPrdLooked = sdo.toObject(WorkInstructionPrdLooked.class);
        WorkInstructionProductLoss workInstructionProductLoss = sdo.toObject(WorkInstructionProductLoss.class);

        WorkInstructionBaseInfo workInstructionBaseInfo = workInstructionRepository.save(workInstruction);

        workInstructionPdPublishedConf.setWorkInstructionBaseInfoId(workInstructionBaseInfo.getWorkInstructionBaseInfoId());
        workInstructionPdPublishedConfRepository.save(workInstructionPdPublishedConf);

        workInstructionMaterialInfo.setWorkInstructionBaseInfoId(workInstructionBaseInfo.getWorkInstructionBaseInfoId());
        workInstructionMaterialInfoRepository.save(workInstructionMaterialInfo);

        workInstructionOptionalAttribute.setWorkInstructionBaseInfoId(workInstructionBaseInfo.getWorkInstructionBaseInfoId());
        workInstructionOptionalAttributeRepository.save(workInstructionOptionalAttribute);

        workInstructionPrdLooked.setWorkInstructionBaseInfoId(workInstructionBaseInfo.getWorkInstructionBaseInfoId());
        workInstructionPrdLookedRepository.save(workInstructionPrdLooked);

        workInstructionProductLoss.setWorkInstructionBaseInfoId(workInstructionBaseInfo.getWorkInstructionBaseInfoId());
        workInstructionProductLossRepository.save(workInstructionProductLoss);

        return workInstructionBaseInfo.getWorkInstructionBaseInfoId();
    }

    @Override
    public Map<String, Object> slicedWorkInstructionBaseInfoList(WorkInstructionBaseInfoQueryRequest queryRequest) {
        QueryDescriptor queryDescriptor = this.sqlExecutorDao.getQuery(QUERY_XML_FILE_PATH, "slicedWorkInstructionBaseInfoList");
        return this.sqlExecutorDao.executeSlicedQuery(queryDescriptor,queryRequest);
    }

    @Override
    public WorkInstructionVo getWorkInstructionVoById(String id) {
        WorkInstructionVo workInstructionVo = new WorkInstructionVo();

        Stream.of(
                   workInstructionRepository.findOne(id),
                   workInstructionPdPublishedConfRepository.findByWorkInstructionBaseInfoId(id),
                   workInstructionMaterialInfoRepository.findByWorkInstructionBaseInfoId(id),
                   workInstructionOptionalAttributeRepository.findByWorkInstructionBaseInfoId(id),
                   workInstructionPrdLookedRepository.findByWorkInstructionBaseInfoId(id),
                   workInstructionProductLossRepository.findByWorkInstructionBaseInfoId(id)
                ).forEach(source -> MyBaseUtil.mergeObjects(workInstructionVo, source));
        return workInstructionVo;
    }

    @Override
    @Transactional(propagation = Propagation.REQUIRES_NEW)
    public void deleteWorkInstructionById(String id) {
        workInstructionPdPublishedConfRepository.deleteByWorkInstructionBaseInfoId(id);
        workInstructionMaterialInfoRepository.deleteByWorkInstructionBaseInfoId(id);
        workInstructionOptionalAttributeRepository.deleteByWorkInstructionBaseInfoId(id);
        workInstructionPrdLookedRepository.deleteByWorkInstructionBaseInfoId(id);
        workInstructionProductLossRepository.deleteByWorkInstructionBaseInfoId(id);
        workInstructionRepository.delete(id);
    }

    @Override
    @Transactional
    public void deleteWorkInstructionByIds(List<String> ids) {
        ids.forEach(this::deleteWorkInstructionById);
    }
}
